/*
 * OPALE is a scientific library under LGPL. Its main goal is to
 * develop mathematical tools for any scientist.
 *
 * Copyright (C) 2002 Opale Group
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * You can visit the web site http://opale.tuxfamily.org to obtain more
 * informations about this program and/or to contact the authors by mail
 * developers@opale.tuxfamily.org.
 */




package opale.m2d;
import opale.tools.*;
import java.io.*;

/**
* Cette classe hritent de la classe de base <code>Vector2D</code> et implmente
* l'interface <code>GObject2D</code>
* @author O.C.
* @since Opale-2d 0.11
*/

public class GVector2D extends Vector2D implements GObject2D,Cloneable
{
//membres privs
private InfoVector info; //Info sur le vecteur
private GPoint2D origin;	// l'origine du vecteur ATTENTION:prive car on ne doit pas modifier son repere.

/**
* Constructeur par dfaut.
*/
public GVector2D()
	{
	if (Debug.On) Debug.print("ici dan GVector2D");
	info = new InfoVector();
	origin = new GPoint2D();
	}

/**
* Le constructeur par dfaut
* Construit et initialise le vecteur avec les coomposantes (0; 0) et avec les informations par dfaut.  
* @param OIJ rep, le repre de dfinition du vecteur
*/
public GVector2D(OIJ rep)
	{
	super(rep);
	info = new InfoVector();
	origin = new GPoint2D(rep);
	}

/**
* Construit et initialise le vecteur avec les composantes (x; y) et avec les informations par dfaut.  
* @param OIJ rep, le repre de dfinition du vecteur
* @param double x,y les composantes du vecteur.
*/
public GVector2D(OIJ rep,double x, double y)
	{
	super(rep,x,y);
	info = new InfoVector();
	origin = new GPoint2D(rep);
	}


/**
* Construit et initialise le vecteur avec un autre vecteur et avec un nom.  
* @param Vector2D v le vecteur  copier.
* @param String nom, le nom de l'objet.
*/
public GVector2D(Vector2D v, String nom)
	{
	super(v);
	origin = new GPoint2D(v.getOIJ());
	info = new InfoVector();
	info.setName(nom);
	}

/**
* Le constructeur de copie
* @param GVector2D v, le vecteur  copier
*/
public GVector2D(GVector2D v)
	{
	super(v);
	origin = (GPoint2D) v.getOrigin().clone();
	info = new InfoVector((InfoVector) v.getInfo());
	}

public void draw(Pen2D c)
	{
	if (info.isVisible())
		{
		double ox = getOrigin().x;
		double oy = getOrigin().y;
		c.setColor(info.getColor());
		if (info.symbolArrow == 0)
			{
			c.drawLine(ox,oy,ox+x,oy+y);
			c.drawArrow(ox,oy,ox+x,oy+y,info.size);
			}
		//((InfoPoint) origin.getInfo()).setSymbol(info.symbol);
		}
	}
	
public InfoObject getInfo()
	{
	return info;
	}
/**
* Cre un nouvel objet de mme classe et de mme contenu.
* @return Object un clone de l'objet.
* @exception  OutOfMemoryError s'il n'y a pas assez de mmoire.
* @see        java.lang.Cloneable
*/
public Object clone()
	{
	try
		{
		GVector2D v = (GVector2D) super.clone();
		v.info = (InfoVector) info.clone();
		v.origin = (GPoint2D) origin.clone();
		return v;
		}
	catch (InternalError e) // ne devrait jamais arriver
		{
	    	throw e;
		}
    }


/**
* Renvoie un clone de l'origine du vecteur.
* @return GPoint2D l'origine.
*/
public GPoint2D getOrigin()
	{
	return (GPoint2D) origin.clone();
	}

/**
* Fixe l'origine du vecteur.
* @param double x,y l'origine.
*/
public void setOrigin(double x, double y)
	{
	origin.setLocation(x,y);
	}

/**
* Surcharge de setOIJ(OIJ rep) de Object2D pour changer a la fois le repere du
vecteur et de l'origine.
* @param OIJ rep, le nouveau repere.
*/
public void setOIJ(OIJ rep)
	{
	super.setOIJ(rep);
	origin.setOIJ(rep);
	}

/*public void writeFile(WFile f, Pack2D p)
	{
	f.writeln("{");
	f.writeln("Origine "+origin.x+" "+origin.y);
	f.writeln("X " +x);
	f.writeln("Y " +y);
	f.writeln("}");
	}	
*/
/**
* Effectue un changement de repere du vecteur.
* @param OIJ rep, le nouveau repere.
*/
public void changeOIJ(OIJ rep)
	{
	origin.changeOIJ(rep);
	super.changeOIJ(rep);
	}
	
	
	
public int readKeyWord(String word, StreamReader f, OpaleSet p) throws java.io.IOException,InvalidFormatException 
	{

	if ( word.equals("INFO"))
		{
		info.read(f,p);
		return 0;
		}
	else if ( word.equals("ORIGIN"))
		{
		int type;
		origin.x = f.nextDouble();
		origin.y = f.nextDouble();
		return 0;
		}
	else return super.readKeyWord(word,f,p);
	}

public void writeMore(PrintWriter f, OpaleSet p) throws InvalidFormatException
	{
	info.write(f,p);
	f.println("ORIGIN "+origin.x+" " +origin.y);
	super.writeMore(f,p);
	}	
	
public double x(double t) { return origin.x; }
public double y(double t) {  return origin.y; };

}

